@@ -1,9 +1,17 @@
+#
+# Copyright (C) 2020-2023 by Ilkka Tuohela <hile@iki.fi>
+#
+# SPDX-License-Identifier: BSD-3-Clause
+#
 """
 Loader for configuration files in yaml format
 """
+from pathlib import Path
+from typing import Union
 
 import yaml
 
+from ..constants import DEFAULT_ENCODING
 from ..exceptions import ConfigurationError
 from .directory import ConfigurationFileDirectory
 from .file import ConfigurationFile
@@ -18,16 +26,16 @@ class YamlConfiguration(ConfigurationFile):
     """
     Configuration parser for yaml configuration files
     """
-    encoding = 'utf-8'
+    encoding = DEFAULT_ENCODING
 
-    def load(self, path):
+    def load(self, path: Union[str, Path]) -> None:
         """
         Load specified YAML configuration file
         """
         path = self.__check_file_access__(path)
 
         try:
-            with path.open('r', encoding='utf-8') as handle:
+            with path.open('r', encoding=self.encoding) as handle:
                 self.parse_data(yaml.safe_load(handle))
         except Exception as error:
             raise ConfigurationError(f'Error loading {path}: {error}') from error
