@@ -1,7 +1,11 @@
+#
+# Copyright (C) 2020-2023 by Ilkka Tuohela <hile@iki.fi>
+#
+# SPDX-License-Identifier: BSD-3-Clause
+#
 """
 System information lookup functions specific to MacOS Darwin
 """
-
 from inflection import underscore
 
 from ..subprocess import run_command_lineoutput
@@ -13,11 +17,11 @@ class SoftwareVersion:
     """
     __data__ = {}
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return f'{self.product_name} {self.product_version} {self.build_version}'
 
     @staticmethod
-    def __get_sw_vers_data__():
+    def __get_sw_vers_data__() -> dict:
         """
         Run sw_vers and output it's data
         """
@@ -29,7 +33,7 @@ class SoftwareVersion:
         return data
 
     @property
-    def product_name(self):
+    def product_name(self) -> str:
         """
         Return MacOS product name
         """
@@ -38,7 +42,7 @@ class SoftwareVersion:
         return self.__data__['product_name']
 
     @property
-    def product_version(self):
+    def product_version(self) -> str:
         """
         Return MacOS product version
         """
@@ -47,7 +51,7 @@ class SoftwareVersion:
         return self.__data__['product_version']
 
     @property
-    def build_version(self):
+    def build_version(self) -> str:
         """
         Return MacOS build version
         """
